@@ -34,7 +34,7 @@ struct _GnomePrintFilterPrivate {
 
 	GPtrArray *meta;
 	GPtrArray *meta_bin;
-	GnomePrintMeta *meta_bin_last;
+	GnomePrintContext *meta_bin_last;
 
 	gboolean suppress_empty_pages;
 };
@@ -45,7 +45,8 @@ enum {
 	PROP_DESCRIPTION,
 	PROP_SUPPRESS_EMPTY_PAGES,
 	PROP_CONTEXT,
-	PROP_TRANSFORM
+	PROP_TRANSFORM,
+	PROP_FILTERS
 };
 
 enum {
@@ -65,6 +66,9 @@ static void
 gnome_print_filter_get_property (GObject *object, guint n, GValue *v, GParamSpec *pspec)
 {
 	GnomePrintFilter *f = (GnomePrintFilter *) object;
+	guint i;
+	GValueArray *va;
+	GValue vd = {0,};
 
 	switch (n) {
 	case PROP_NAME:		g_value_set_string  (v, _(FILTER_NAME));		break;
@@ -74,20 +78,26 @@ gnome_print_filter_get_property (GObject *object, guint n, GValue *v, GParamSpec
 		break;
 	case PROP_CONTEXT: g_value_set_object (v, f->priv->pc ); break;
 	case PROP_TRANSFORM: 
-		{
-			guint i;
-			GValueArray *va = g_value_array_new (6);
-			GValue vd = {0,};
-
-			g_value_init (&vd, G_TYPE_DOUBLE);
-			for (i = 0; i < 6; i++) {
-				g_value_set_double (&vd, f->priv->transform[i]);
-				g_value_array_append (va, &vd);
-			}
-			g_value_unset (&vd);
-			g_value_set_boxed (v, va);
-			g_value_array_free (va);
+		va = g_value_array_new (6);
+		g_value_init (&vd, G_TYPE_DOUBLE);
+		for (i = 0; i < 6; i++) {
+			g_value_set_double (&vd, f->priv->transform[i]);
+			g_value_array_append (va, &vd);
 		}
+		g_value_unset (&vd);
+		g_value_set_boxed (v, va);
+		g_value_array_free (va);
+		break;
+	case PROP_FILTERS:
+		va = g_value_array_new (0);
+		g_value_init (&vd, G_TYPE_OBJECT);
+		for (i = 0; i < (f->priv->filters ? f->priv->filters->len : 0); i++) {
+			g_value_set_object (&vd, g_ptr_array_index (f->priv->filters, i));
+			g_value_array_append (va, &vd);
+		}
+		g_value_unset (&vd);
+		g_value_set_boxed (v, va);
+		g_value_array_free (va);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (object, n, pspec);
 	}
@@ -99,6 +109,8 @@ gnome_print_filter_set_property (GObject *object, guint n, const GValue *v,
 {
 	GnomePrintFilter *f = (GnomePrintFilter *) object;
 	gboolean b;
+	GValueArray *va = NULL;
+	guint i;
 
 	switch (n) {
 	case PROP_SUPPRESS_EMPTY_PAGES:
@@ -109,21 +121,24 @@ gnome_print_filter_set_property (GObject *object, guint n, const GValue *v,
 		}
 		break;
 	case PROP_TRANSFORM:
-		{
-			GValueArray *a = g_value_get_boxed (v);
-
-			if (!a || !a->n_values)
-				art_affine_identity (f->priv->transform);
-			else {
-				guint i;
-
-				for (i = 0; (i < a->n_values) && (i < 6); i++)
-					f->priv->transform[i] = g_value_get_double (
-							g_value_array_get_nth (a, i));
-			}
-		}
+		va = g_value_get_boxed (v);
+		if (!va || !va->n_values)
+			art_affine_identity (f->priv->transform);
+		else
+			for (i = 0; (i < va->n_values) && (i < 6); i++)
+				f->priv->transform[i] = g_value_get_double (
+						g_value_array_get_nth (va, i));
 		gnome_print_filter_changed (f);
 		break;
+	case PROP_FILTERS:
+		va = g_value_get_boxed (v);
+		while (gnome_print_filter_count_filters (f))
+			gnome_print_filter_remove_filter (f, NULL);
+		for (i = 0; i < (va ? va->n_values : 0); i++)
+			gnome_print_filter_add_filter (f,
+				GNOME_PRINT_FILTER (g_value_get_object (
+					g_value_array_get_nth (va, i))));
+		break;
 	case PROP_CONTEXT:
 		if (f->priv->pc)
 			g_object_remove_weak_pointer (G_OBJECT (f->priv->pc),
@@ -140,25 +155,21 @@ gnome_print_filter_set_property (GObject *object, guint n, const GValue *v,
 #define GPFGC(f) GNOME_PRINT_FILTER_GET_CLASS(f)
 
 void
-gnome_print_filter_reset (GnomePrintFilter *f)
+gnome_print_filter_clear_data (GnomePrintFilter *f)
 {
-	guint i, n;
+	guint i;
 
 	g_return_if_fail (GNOME_IS_PRINT_FILTER (f));
 
-	n = gnome_print_filter_count_filters (f);
-	for (i = 0; i < n; i++)
-		gnome_print_filter_reset (gnome_print_filter_get_filter (f, i));
-
-	n = gnome_print_filter_count_successors (f);
-	for (i = 0; i < n; i++)
-		gnome_print_filter_reset (gnome_print_filter_get_successor (f, i));
-	for (i = 1; i < n; i++)
-		gnome_print_meta_reset (g_ptr_array_index (f->priv->meta, i - 1));
-
-	if (f->priv->pc) {
-		g_object_unref (G_OBJECT (f->priv->pc));
-		f->priv->pc = NULL;
+	if (f->priv->meta_bin_last) {
+		g_object_unref (f->priv->meta_bin_last);
+		f->priv->meta_bin_last = NULL;
+	}
+	if (f->priv->meta_bin) {
+		for (i = 0; i < f->priv->meta_bin->len; i++)
+			g_object_unref (G_OBJECT (f->priv->meta_bin->pdata[i]));
+		g_ptr_array_free (f->priv->meta_bin, TRUE);
+		f->priv->meta_bin = NULL;
 	}
 }
 
@@ -194,17 +205,13 @@ beginpage_impl (GnomePrintFilter *f, GnomePrintContext *pc, const guchar *name)
 	/* Check if this filter doesn't have successors. */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
-			return gnome_print_beginpage_real (pc, name);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_beginpage_real (pc, name);
 		if (!f->priv->bin->priv->meta_bin_last)
 			f->priv->bin->priv->meta_bin_last =
 				g_object_new (GNOME_TYPE_PRINT_META, NULL);
-		C (gnome_print_beginpage_real (
-					GNOME_PRINT_CONTEXT (f->priv->bin->priv->meta_bin_last), name));
-		return GNOME_PRINT_OK;
+		return gnome_print_beginpage_real (
+							f->priv->bin->priv->meta_bin_last, name);
 	}
 
 	/* This filter has successors */
@@ -225,10 +232,14 @@ showpage_impl (GnomePrintFilter *f)
 	guint i, n;
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
-	g_return_val_if_fail (gnome_print_filter_haspage (f), GNOME_PRINT_ERROR_BADVALUE);
+
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
 
 	/* Check if this filter has filters inside. */
 	g_object_get (G_OBJECT (f->priv->pc), "filter", &f_old, NULL);
+	if (f_old)
+		g_object_ref (G_OBJECT (f_old));
 	n = gnome_print_filter_count_filters (f);
 	if (n) {
 		gnome_print_filter_showpage (
@@ -253,20 +264,19 @@ showpage_impl (GnomePrintFilter *f)
 	if (!n) {
 		if (!f->priv->bin) {
 			gnome_print_showpage_real (f->priv->pc);
-			return GNOME_PRINT_OK;
+			goto reset_filter;
 		}
 		n = gnome_print_filter_count_successors (f->priv->bin);
 		if (!n) {
 			gnome_print_showpage_real (f->priv->pc);
-			return GNOME_PRINT_OK;
+			goto reset_filter;
 		}
-		gnome_print_showpage_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last));
+		gnome_print_showpage_real (f->priv->bin->priv->meta_bin_last);
 		for (i = 0; i < n; i++) {
 			g_object_set (G_OBJECT (f->priv->pc), "filter",
 					gnome_print_filter_get_successor (f->priv->bin, i), NULL);
-			gnome_print_meta_render (f->priv->bin->priv->meta_bin_last,
-						f->priv->pc);
+			gnome_print_meta_render (
+				GNOME_PRINT_META (f->priv->bin->priv->meta_bin_last), f->priv->pc);
 		}
 		g_object_unref (G_OBJECT (f->priv->bin->priv->meta_bin_last));
 		f->priv->bin->priv->meta_bin_last = NULL;
@@ -288,7 +298,7 @@ showpage_impl (GnomePrintFilter *f)
 
 reset_filter :
 	g_object_set (G_OBJECT (f->priv->pc), "filter", f_old, NULL);
-	if (NULL != f_old)
+	if (f_old)
 		g_object_unref (G_OBJECT (f_old));
 	return GNOME_PRINT_OK;
 }
@@ -300,6 +310,9 @@ gsave_impl (GnomePrintFilter *f)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	/* Check if this filter has filters inside. */
 	n = gnome_print_filter_count_filters (f);
 	if (n) {
@@ -313,14 +326,9 @@ gsave_impl (GnomePrintFilter *f)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_gsave_real (f->priv->pc);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_gsave_real (f->priv->pc);
-		C (gnome_print_gsave_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last)));
-		return GNOME_PRINT_OK;
+		return gnome_print_gsave_real (f->priv->bin->priv->meta_bin_last);
 	}
 
 	/* This filter has successors */
@@ -338,6 +346,9 @@ grestore_impl (GnomePrintFilter *f)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	/* Check if this filter has filters inside. */
 	n = gnome_print_filter_count_filters (f);
 	if (n) {
@@ -351,14 +362,9 @@ grestore_impl (GnomePrintFilter *f)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_grestore_real (f->priv->pc);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_grestore_real (f->priv->pc);
-		C (gnome_print_grestore_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last)));
-		return GNOME_PRINT_OK;
+		return gnome_print_grestore_real (f->priv->bin->priv->meta_bin_last);
 	}
 
 	/* This filter has successors */
@@ -369,7 +375,7 @@ grestore_impl (GnomePrintFilter *f)
 	return GNOME_PRINT_OK;
 }
 
-gint
+static gint
 clip_impl (GnomePrintFilter *f, const ArtBpath *bpath, ArtWindRule r)
 {
 	ArtBpath *b;
@@ -377,6 +383,9 @@ clip_impl (GnomePrintFilter *f, const ArtBpath *bpath, ArtWindRule r)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	b = art_bpath_affine_transform (bpath, f->priv->transform);
 
 	/* Check if this filter has filters inside. */
@@ -393,19 +402,11 @@ clip_impl (GnomePrintFilter *f, const ArtBpath *bpath, ArtWindRule r)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin) {
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			gnome_print_clip_bpath_rule_real (f->priv->pc, b, r);
-			art_free (b);
-			return GNOME_PRINT_OK;
-		}
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n) {
-			gnome_print_clip_bpath_rule_real (f->priv->pc, b, r);
-			art_free (b);
-			return GNOME_PRINT_OK;
-		}
-		gnome_print_clip_bpath_rule_real (
-				GNOME_PRINT_CONTEXT (f->priv->bin->priv->meta_bin_last), b, r);
+		else
+			gnome_print_clip_bpath_rule_real (
+											f->priv->bin->priv->meta_bin_last, b, r);
 		art_free (b);
 		return GNOME_PRINT_OK;
 	}
@@ -427,6 +428,9 @@ fill_impl (GnomePrintFilter *f, const ArtBpath *bpath, ArtWindRule r)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	b = art_bpath_affine_transform (bpath, f->priv->transform);
 
 	/* Check if this filter has filters inside. */
@@ -443,19 +447,10 @@ fill_impl (GnomePrintFilter *f, const ArtBpath *bpath, ArtWindRule r)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin) {
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			gnome_print_fill_bpath_rule_real (f->priv->pc, b, r);
-			art_free (b);
-			return GNOME_PRINT_OK;
-		}
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n) {
-			gnome_print_fill_bpath_rule_real (f->priv->pc, b, r);
-			art_free (b);
-			return GNOME_PRINT_OK;
-		}
-		gnome_print_fill_bpath_rule_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), b, r);
+		else
+			gnome_print_fill_bpath_rule_real (f->priv->bin->priv->meta_bin_last, b, r);
 		art_free (b);
 		return GNOME_PRINT_OK;
 	}
@@ -475,6 +470,9 @@ stroke_impl (GnomePrintFilter *f, const ArtBpath *bpath)
 	ArtBpath *b;
 	guint i, n;
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	b = art_bpath_affine_transform (bpath, f->priv->transform);
 
 	/* Check if this filter has filters inside. */
@@ -491,19 +489,10 @@ stroke_impl (GnomePrintFilter *f, const ArtBpath *bpath)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin) {
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			gnome_print_stroke_bpath_real (f->priv->pc, b);
-			art_free (b);
-			return GNOME_PRINT_OK;
-		}
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n) {
-			gnome_print_stroke_bpath_real (f->priv->pc, b);
-			art_free (b);
-			return GNOME_PRINT_OK;
-		}
-		gnome_print_stroke_bpath_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), b);
+		else
+			gnome_print_stroke_bpath_real (f->priv->bin->priv->meta_bin_last, b);
 		art_free (b);
 		return GNOME_PRINT_OK;
 	}
@@ -525,6 +514,9 @@ image_impl (GnomePrintFilter *f, const gdouble *affine, const guchar *p, gint w,
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	art_affine_multiply (a, affine, f->priv->transform);
 
 	/* Check if this filter has filters inside. */
@@ -541,16 +533,10 @@ image_impl (GnomePrintFilter *f, const gdouble *affine, const guchar *p, gint w,
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
-			return gnome_print_image_transform_real (f->priv->pc,
-					a, p, w, h, r, c);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_image_transform_real (f->priv->pc,
-					a, p, w, h, r, c);
-		C (gnome_print_image_transform_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), a, p, w, h, r, c));
-		return GNOME_PRINT_OK;
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
+			return gnome_print_image_transform_real (f->priv->pc, a, p, w, h, r, c);
+		return gnome_print_image_transform_real (
+										f->priv->bin->priv->meta_bin_last, a, p, w, h, r, c);
 	}
 
 	/* This filter has successors */
@@ -569,6 +555,9 @@ glyphlist_impl (GnomePrintFilter *f, const gdouble *affine, GnomeGlyphList *l)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	art_affine_multiply (a, affine, f->priv->transform);
 
 	/* Check if this filter has filters inside. */
@@ -584,14 +573,10 @@ glyphlist_impl (GnomePrintFilter *f, const gdouble *affine, GnomeGlyphList *l)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_glyphlist_transform_real (f->priv->pc, a, l);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_glyphlist_transform_real (f->priv->pc, a, l);
-		C (gnome_print_glyphlist_transform_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), a, l));
-		return GNOME_PRINT_OK;
+		return gnome_print_glyphlist_transform_real (
+														f->priv->bin->priv->meta_bin_last, a, l);
 	}
 
 	/* This filter has successors */
@@ -609,6 +594,9 @@ setrgbcolor_impl (GnomePrintFilter *f, gdouble r, gdouble g, gdouble b)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	/* Check if this filter has filters inside. */
 	n = gnome_print_filter_count_filters (f);
 	if (n) {
@@ -622,14 +610,10 @@ setrgbcolor_impl (GnomePrintFilter *f, gdouble r, gdouble g, gdouble b)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_setrgbcolor_real (f->priv->pc, r, g, b);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_setrgbcolor_real (f->priv->pc, r, g, b);
-		C (gnome_print_setrgbcolor_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), r, g, b));
-		return GNOME_PRINT_OK;
+		return gnome_print_setrgbcolor_real (
+						f->priv->bin->priv->meta_bin_last, r, g, b);
 	}
 
 	/* This filter has successors */
@@ -640,13 +624,16 @@ setrgbcolor_impl (GnomePrintFilter *f, gdouble r, gdouble g, gdouble b)
 	return GNOME_PRINT_OK;
 }
 
-gint
+static gint
 setopacity_impl (GnomePrintFilter *f, gdouble o)
 {
 	guint i, n;
 	
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	/* Check if this filter has filters inside. */
 	n = gnome_print_filter_count_filters (f);
 	if (n) {
@@ -660,14 +647,9 @@ setopacity_impl (GnomePrintFilter *f, gdouble o)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_setopacity_real (f->priv->pc, o);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_setopacity_real (f->priv->pc, o);
-		C (gnome_print_setopacity_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), o));
-		return GNOME_PRINT_OK;
+		return gnome_print_setopacity_real (f->priv->bin->priv->meta_bin_last, o);
 	}
 
 	/* This filter has successors */
@@ -685,6 +667,9 @@ setlinewidth_impl (GnomePrintFilter *f, gdouble w)
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_BADVALUE);
 
+	if (!gnome_print_filter_haspage (f))
+		return GNOME_PRINT_OK;
+
 	/* Check if this filter has filters inside. */
 	n = gnome_print_filter_count_filters (f);
 	if (n) {
@@ -698,14 +683,9 @@ setlinewidth_impl (GnomePrintFilter *f, gdouble w)
 	/* Check if this filter has successors */
 	n = gnome_print_filter_count_successors (f);
 	if (!n) {
-		if (!f->priv->bin)
+		if (!f->priv->bin || !gnome_print_filter_count_successors (f->priv->bin))
 			return gnome_print_setlinewidth_real (f->priv->pc, w);
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		if (!n)
-			return gnome_print_setlinewidth_real (f->priv->pc, w);
-		C (gnome_print_setlinewidth_real (GNOME_PRINT_CONTEXT (
-						f->priv->bin->priv->meta_bin_last), w));
-		return GNOME_PRINT_OK;
+		return gnome_print_setlinewidth_real (f->priv->bin->priv->meta_bin_last, w);
 	}
 
 	/* This filter has successors */
@@ -725,51 +705,39 @@ flush_impl (GnomePrintFilter *f)
 	if (n) {
 		for (i = 0; i < n; i++)
 			gnome_print_filter_flush (gnome_print_filter_get_filter (f, i));
-		return;
 	}
 	n = gnome_print_filter_count_successors (f);
 	if (n) {
 		for (i = 0; i < n; i++)
 			gnome_print_filter_flush (gnome_print_filter_get_successor (f, i));
-		return;
 	}
 	if (f->priv->bin) {
 		n = gnome_print_filter_count_successors (f->priv->bin);
 		for (i = 0; i < n; i++)
 			gnome_print_filter_flush (
 					gnome_print_filter_get_successor (f->priv->bin, i));
-		return;
 	}
 }
 
 static void
 reset_impl (GnomePrintFilter *f)
 {
-	guint i, n;
+	guint i;
 
-	n = gnome_print_filter_count_filters (f);
-	if (n) {
-		for (i = 0; i < n; i++)
-			gnome_print_filter_reset (gnome_print_filter_get_filter (f, i));
-		return;
-	}
-	n = gnome_print_filter_count_successors (f);
-	if (n) {
-		for (i = 0; i < n; i++)
-			gnome_print_filter_reset (gnome_print_filter_get_successor (f, i));
-		return;
-	}
-	if (f->priv->bin) {
-		n = gnome_print_filter_count_successors (f->priv->bin);
-		for (i = 0; i < n; i++)
-			gnome_print_filter_reset (
-					gnome_print_filter_get_successor (f->priv->bin, i));
-		return;
-	}
-	if (f->priv->pc) {
-		g_object_unref (G_OBJECT (f->priv->pc));
-		f->priv->pc = NULL;
-	}
+	gnome_print_filter_clear_data (f);
+
+	for (i = gnome_print_filter_count_filters (f); i > 0; i--)
+		gnome_print_filter_reset (gnome_print_filter_get_filter (f, i - 1));
+
+	for (i = gnome_print_filter_count_successors (f); i > 0; i--)
+		gnome_print_filter_reset (gnome_print_filter_get_successor (f, i - 1));
+	for (i = gnome_print_filter_count_successors (f); i > 1; i--)
+		gnome_print_meta_reset (g_ptr_array_index (f->priv->meta, i - 2));
+	if (f->priv->bin)
+		for (i = gnome_print_filter_count_successors (f->priv->bin); i > 0; i--)
+			gnome_print_filter_reset (gnome_print_filter_get_successor (f->priv->bin,
+															i - 1));
+	g_object_set (G_OBJECT (f), "context", NULL, NULL);
 }
 
 void
@@ -812,13 +780,10 @@ gnome_print_filter_finalize (GObject *object)
 {
 	GnomePrintFilter *f = (GnomePrintFilter *) object;
 
+	gnome_print_filter_clear_data (f);
 	gnome_print_filter_remove_filters (f);
 
 	if (f->priv) {
-		if (f->priv->meta_bin_last) {
-			g_object_unref (f->priv->meta_bin_last);
-			f->priv->meta_bin_last = NULL;
-		}
 		while (f->priv->predecessors)
 			gnome_print_filter_remove_predecessor (f,
 				g_ptr_array_index (f->priv->predecessors, 0));
@@ -837,16 +802,8 @@ gnome_print_filter_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-void
-gnome_print_filter_flush (GnomePrintFilter *f)
-{
-	g_return_if_fail (GNOME_IS_PRINT_FILTER (f));
-
-	if (GPFGC (f)->flush) GPFGC (f)->flush (f);
-}
-
 static void
-transform_value_set_default (GParamSpec *pspec, GValue *v)
+param_transform_set_default (GParamSpec *pspec, GValue *v)
 {
 	GValueArray *va = g_value_array_new (6);
 	GValue vd = {0,};
@@ -864,6 +821,69 @@ transform_value_set_default (GParamSpec *pspec, GValue *v)
 	g_value_array_free (va);
 }
 
+struct {
+	GParamSpecValueArray parent_instance;
+} GnomePrintFilterParamTransform;
+
+static void
+param_transform_init (GParamSpec *pspec)
+{
+	GParamSpecValueArray *pspec_va = (GParamSpecValueArray *) pspec;
+
+	pspec_va->element_spec = g_param_spec_double ("transform_item",
+		_("Transform item"), _("Transform item"), -G_MAXDOUBLE, G_MAXDOUBLE, 0.,
+		G_PARAM_READWRITE);
+	g_param_spec_ref (pspec_va->element_spec);
+	g_param_spec_sink (pspec_va->element_spec);
+}
+
+static gint
+param_transform_cmp (GParamSpec *pspec, const GValue *value1,
+								const GValue *value2)
+{
+	GValueArray *value_array1 = g_value_get_boxed (value1);
+	GValueArray *value_array2 = g_value_get_boxed (value2);
+
+	if (!value_array1 || !value_array2)
+		return value_array2 ? -1 : value_array1 != value_array2;
+	if (value_array1->n_values != value_array2->n_values)
+		return value_array1->n_values < value_array2->n_values ? -1 : 1;
+	else {
+		guint i;
+		
+		for (i = 0; i < value_array1->n_values; i++) {
+			GValue *element1 = value_array1->values + i;
+			GValue *element2 = value_array2->values + i;
+			gint cmp;
+
+			if (G_VALUE_TYPE (element1) != G_VALUE_TYPE (element2))
+				return G_VALUE_TYPE (element1) < G_VALUE_TYPE (element2) ? -1 : 1;
+			cmp = g_param_values_cmp (((GParamSpecValueArray *) pspec)->element_spec,
+				element1, element2);
+			if (cmp)
+				return cmp;
+		}
+		return 0;
+	}
+}
+
+static GType
+gnome_print_filter_param_transform_get_type (void)
+{
+	static GType type;
+	if (G_UNLIKELY (type) == 0) {
+		static GParamSpecTypeInfo pspec_info = {
+			sizeof (GnomePrintFilterParamTransform), 0,
+			param_transform_init,
+			0xdeadbeef, NULL, param_transform_set_default, NULL,
+			param_transform_cmp
+		};
+		pspec_info.value_type = G_TYPE_VALUE_ARRAY;
+		type = g_param_type_register_static ("GnomePrintFilterParamTransform", &pspec_info);
+	}
+	return type;
+}
+
 static void
 gnome_print_filter_class_init (GnomePrintFilterClass *filter_class)
 {
@@ -906,11 +926,13 @@ gnome_print_filter_class_init (GnomePrintFilterClass *filter_class)
 	g_object_class_install_property (object_class, PROP_CONTEXT,
 		g_param_spec_object ("context", _("Context"), _("Context"),
 			GNOME_TYPE_PRINT_CONTEXT, G_PARAM_READWRITE));
-	pspec = g_param_spec_value_array ("transform", _("Transform"), _("Transform"),
-				g_param_spec_double ("transform_item", _("Transform item"),
-					_("Transform item"), -G_MAXDOUBLE, G_MAXDOUBLE, 0.,
-					G_PARAM_READWRITE), G_PARAM_READWRITE);
-	G_PARAM_SPEC_GET_CLASS (pspec)->value_set_default = transform_value_set_default;
+	g_object_class_install_property (object_class, PROP_FILTERS,
+		g_param_spec_value_array ("filters", _("Filters"), _("Filters"),
+			g_param_spec_object ("filter", _("Filter"), _("Filter"),
+				GNOME_TYPE_PRINT_FILTER, G_PARAM_READWRITE), G_PARAM_READWRITE));
+	pspec = g_param_spec_internal (
+		gnome_print_filter_param_transform_get_type (),
+		"transform", _("Transform"), _("Transform"), G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_TRANSFORM, pspec);
 
 	signals[CHANGED] = g_signal_new ("changed",
@@ -1125,9 +1147,9 @@ static gchar *
 gnome_print_filter_description_rec (GnomePrintFilter *f, const gchar *in, 
 		gboolean brackets)
 {
-	gchar *u, *t, *buf;
-	guint i, n;
-	GParamSpec **props, **p;
+	gchar *u, *t = NULL, *buf;
+	guint i, n, n_props, j;
+	GParamSpec **props;
 
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), NULL);
 
@@ -1139,51 +1161,56 @@ gnome_print_filter_description_rec (GnomePrintFilter *f, const gchar *in,
 			buf = g_strdup_printf ("%s", G_OBJECT_TYPE_NAME (f));
 		else
 			buf = g_strdup_printf ("%s %s", in, G_OBJECT_TYPE_NAME (f));
-	props = g_object_class_list_properties (G_OBJECT_GET_CLASS (f), NULL);
-	for (p = props; *p; p++) {
-		GParamSpec *pspec = *p;
+	props = g_object_class_list_properties (G_OBJECT_GET_CLASS (f), &n_props);
+	for (j = 0; j < n_props; j++) {
 		GValue v = {0,};
 
-		if (!(pspec->flags & G_PARAM_WRITABLE))
+		/*
+		 * We only save writable parameters. We skip the parameter 
+		 * 'filters'.
+		 */
+		if (!(props[j]->flags & G_PARAM_WRITABLE))
+			continue;
+		if (!strcmp (props[j]->name, "filters"))
 			continue;
 
-		g_value_init (&v, G_PARAM_SPEC_VALUE_TYPE (pspec));
-		g_object_get_property (G_OBJECT (f), pspec->name, &v);
-		if (g_param_value_defaults (pspec, &v)) {
+		g_value_init (&v, G_PARAM_SPEC_VALUE_TYPE (props[j]));
+		g_object_get_property (G_OBJECT (f), props[j]->name, &v);
+		if (g_param_value_defaults (props[j], &v)) {
 			g_value_unset (&v);
 			continue;
 		}
-		if (G_IS_PARAM_SPEC_STRING (pspec)) {
-			t = g_strdup_printf ("%s %s='%s'", buf, pspec->name,
+		if (G_IS_PARAM_SPEC_STRING (props[j])) {
+			t = g_strdup_printf ("%s %s='%s'", buf, props[j]->name,
 					g_value_get_string (&v));
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_INT (pspec)) {
-			t = g_strdup_printf ("%s %s=%i", buf, pspec->name,
+		} else if (G_IS_PARAM_SPEC_INT (props[j])) {
+			t = g_strdup_printf ("%s %s=%i", buf, props[j]->name,
 					g_value_get_int (&v));
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_UINT (pspec)) {
-			t = g_strdup_printf ("%s %s=%i", buf, pspec->name,
+		} else if (G_IS_PARAM_SPEC_UINT (props[j])) {
+			t = g_strdup_printf ("%s %s=%i", buf, props[j]->name,
 					g_value_get_uint (&v));
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_DOUBLE (pspec)) {
-			t = g_strdup_printf ("%s %s=%f", buf, pspec->name,
+		} else if (G_IS_PARAM_SPEC_DOUBLE (props[j])) {
+			t = g_strdup_printf ("%s %s=%f", buf, props[j]->name,
 					g_value_get_double (&v));
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_ULONG (pspec)) {
-			t = g_strdup_printf ("%s %s=%li", buf, pspec->name,
+		} else if (G_IS_PARAM_SPEC_ULONG (props[j])) {
+			t = g_strdup_printf ("%s %s=%li", buf, props[j]->name,
 					g_value_get_ulong (&v));
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_ENUM (pspec)) {
-			t = g_strdup_printf ("%s %s=%i", buf, pspec->name,
+		} else if (G_IS_PARAM_SPEC_ENUM (props[j])) {
+			t = g_strdup_printf ("%s %s=%i", buf, props[j]->name,
 					g_value_get_enum (&v));
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_BOOLEAN (pspec)) {
-			t = g_strdup_printf ("%s %s=%s", buf, pspec->name,
+		} else if (G_IS_PARAM_SPEC_BOOLEAN (props[j])) {
+			t = g_strdup_printf ("%s %s=%s", buf, props[j]->name,
 					g_value_get_boolean (&v) ? "true" : "false");
 			g_free (buf); buf = t;
-		} else if (G_IS_PARAM_SPEC_OBJECT (pspec)) {
+		} else if (G_IS_PARAM_SPEC_OBJECT (props[j])) {
 			/* Do nothing */
-		} else if (G_IS_PARAM_SPEC_VALUE_ARRAY (pspec)) {
+		} else if (props[j]->value_type == G_TYPE_VALUE_ARRAY) {
 			GValueArray *va = g_value_get_boxed (&v);
 
 			if (va) {
@@ -1208,7 +1235,9 @@ gnome_print_filter_description_rec (GnomePrintFilter *f, const gchar *in,
 						t = g_strdup_printf ("%s%f", s, g_value_get_double (val));
 					else if (G_VALUE_HOLDS_STRING (val))
 						t = g_strdup_printf ("%s%s", s, g_value_get_string (val));
-					else {
+					else if (G_VALUE_HOLDS_OBJECT (val)) {
+						/* Do nothing */
+					} else {
 						t = g_strdup_printf ("%s?", s);
 						g_warning ("Implement this!");
 					}
@@ -1219,12 +1248,12 @@ gnome_print_filter_description_rec (GnomePrintFilter *f, const gchar *in,
 					}
 				}
 				if (s) {
-					t = g_strdup_printf ("%s %s=%s", buf, pspec->name, s);
+					t = g_strdup_printf ("%s %s=%s", buf, props[j]->name, s);
 					g_free (buf); g_free (s); buf = t;
 				}
 			}
 		} else
-			g_warning ("Implement %s!", g_type_name (pspec->value_type));
+			g_warning ("Implement %s!", g_type_name (props[j]->value_type));
 		g_value_unset (&v);
 	}
 	g_free (props);
@@ -1311,7 +1340,6 @@ gint
 gnome_print_filter_showpage (GnomePrintFilter *f)
 {
 	g_return_val_if_fail (GNOME_IS_PRINT_FILTER (f), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (gnome_print_filter_haspage (f), GNOME_PRINT_ERROR_UNKNOWN);
 
 	return (GPFGC (f)->showpage) ?
 		GPFGC (f)->showpage (f) : GNOME_PRINT_OK;
@@ -1403,6 +1431,24 @@ gnome_print_filter_setlinewidth (GnomePrintFilter *f, gdouble w)
 		GPFGC (f)->setlinewidth (f, w) : GNOME_PRINT_OK;
 }
 
+void
+gnome_print_filter_reset (GnomePrintFilter *f)
+{
+	g_return_if_fail (GNOME_IS_PRINT_FILTER (f));
+
+	if (GPFGC (f)->reset)
+		GPFGC (f)->reset (f);
+}
+
+void
+gnome_print_filter_flush (GnomePrintFilter *f)
+{
+	g_return_if_fail (GNOME_IS_PRINT_FILTER (f));
+
+	if (GPFGC (f)->flush)
+		GPFGC (f)->flush (f);
+}
+
 guint
 gnome_print_filter_count_filters (GnomePrintFilter *f)
 {
@@ -1425,8 +1471,10 @@ gnome_print_filter_remove_filters (GnomePrintFilter *f)
 {
 	g_return_if_fail (GNOME_IS_PRINT_FILTER (f));
 
+	g_object_freeze_notify (G_OBJECT (f));
 	while (gnome_print_filter_count_filters (f))
 		gnome_print_filter_remove_filter (f, gnome_print_filter_get_filter (f, 0));
+	g_object_thaw_notify (G_OBJECT (f));
 }
 
 void
@@ -1447,6 +1495,7 @@ gnome_print_filter_remove_filter (GnomePrintFilter *f, GnomePrintFilter *fs)
 		g_ptr_array_free (f->priv->filters, TRUE);
 		f->priv->filters = NULL;
 	}
+	g_object_notify (G_OBJECT (f), "filters");
 }
 
 void
@@ -1465,6 +1514,7 @@ gnome_print_filter_add_filter (GnomePrintFilter *f, GnomePrintFilter *fs)
 	if (!f->priv->filters)
 		f->priv->filters = g_ptr_array_new ();
 	g_ptr_array_add (f->priv->filters, fs);
+	g_object_notify (G_OBJECT (f), "filters");
 }
 
 GQuark
