@@ -35,24 +35,35 @@ G_BEGIN_DECLS
 #define GNOME_IS_PRINT_JOB(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), GNOME_TYPE_PRINT_JOB))
 
 typedef struct _GnomePrintJob      GnomePrintJob;
+typedef struct _GnomePrintJobClass GnomePrintJobClass;
 
 #include <libgnomeprint/gnome-print.h>
 
 GType gnome_print_job_get_type (void);
 
 GnomePrintJob *      gnome_print_job_new (GnomePrintConfig *config);
+
+/* Don't forget to unref the objects returned by the following 2 functions. */
 GnomePrintConfig *   gnome_print_job_get_config  (GnomePrintJob *job);
 GnomePrintContext *  gnome_print_job_get_context (GnomePrintJob *job);
 
 gint     gnome_print_job_close (GnomePrintJob *job);
 gint     gnome_print_job_print (GnomePrintJob *job);
-gint     gnome_print_job_print_to_file (GnomePrintJob *job, gchar *output);
 
-gboolean gnome_print_job_get_page_size_from_config (GnomePrintConfig *config, gdouble *width, gdouble *height);
+#ifdef GNOME_PRINT_UNSTABLE_API
+void     gnome_print_job_set_file (GnomePrintJob *job, gchar *input);
+#endif
+
 gint     gnome_print_job_render      (GnomePrintJob *job, GnomePrintContext *ctx);
 gint     gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint page, gboolean pageops);
 gint     gnome_print_job_get_pages   (GnomePrintJob *job);
- 
+
+gint     gnome_print_job_print_to_file (GnomePrintJob *job, gchar *output);
+gboolean gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *height);
+
+/* Deprecated, use gnome_print_config_get_page_size */
+gboolean gnome_print_job_get_page_size_from_config (GnomePrintConfig *config, gdouble *width, gdouble *height);
+
 G_END_DECLS
 
 #endif /* __GNOME_PRINT_JOB_H__ */
