@@ -109,7 +109,7 @@ gint gnome_print_image_transform_real     (GnomePrintContext *, const gdouble *,
 gint gnome_print_glyphlist_transform_real (GnomePrintContext *, const gdouble *, GnomeGlyphList *);
 gint gnome_print_setrgbcolor_real         (GnomePrintContext *, gdouble, gdouble, gdouble);
 gint gnome_print_setopacity_real          (GnomePrintContext *, gdouble);
-gint gnome_print_setlinewidth_real        (GnomePrintContext *, double);
+gint gnome_print_setlinewidth_real        (GnomePrintContext *, gdouble);
 
 typedef struct _GnomePrintLayout         GnomePrintLayout;
 typedef struct _GnomePrintLayoutPage     GnomePrintLayoutPage;
