@@ -367,7 +367,8 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 {
 	GnomePrintPs2 *ps2 = GNOME_PRINT_PS2 (pc);
 	gchar *hex;
-	gint hex_size, r;
+	guchar *row;
+	gint hex_size, r, out_ch;
 	gboolean problem;
 
 	problem  = gnome_print_ps2_puts (ps2, "q" EOL "[");
@@ -392,20 +393,70 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 	if (ch == 1) {
 		problem |= gnome_print_ps2_puts (ps2, "image" EOL);
 	} else {
-		problem |= gnome_print_ps2_fprintf (ps2, "false %d colorimage" EOL, ch);
+		problem |= gnome_print_ps2_fprintf (ps2, "false 3 colorimage" EOL);
 	}
 	g_return_val_if_fail (!problem, GNOME_PRINT_ERROR_UNKNOWN);
 
-	hex = g_new (gchar, gnome_print_encode_hex_wcs (w * ch));
+	out_ch = ch;
+	if (ch == 4) {
+		/* Postscript doesn't support alpha channel.  The 4-channel
+		* colorimage operator takes CMYK, not RGBA!
+		* We flatten image against white background in the loop.
+		*/
+		out_ch = 3;
+		row = g_new (guchar, w * out_ch);
+	}
+
+	hex = g_new (gchar, gnome_print_encode_hex_wcs (w * out_ch));
 
 	for (r = 0; r < h; r++) {
-		hex_size = gnome_print_encode_hex (px + r * rowstride, (guchar *) hex, w * ch);
+	        if (out_ch == ch) {
+			row = px + r * rowstride;
+		} else {
+			const guchar *p = px + r * rowstride;
+			guchar *q = row;
+			int i;
+
+			/* Flatten.
+			 * Blend against white background is:
+			 * 	new_color = color * alpha + (1 - alpha)
+			 * With 8-bit data that is:
+			 * 	new_color = color * alpha / 255 + (255 - alpha)
+			 */
+			for (i = 0; i < w; i++) {
+				unsigned int alpha = p[3];
+				unsigned int one_minus_alpha = 255 - alpha;
+
+				/* special case common cases */
+				if (alpha == 255) {
+					*q++ = *p++;
+					*q++ = *p++;
+					*q++ = *p++;
+					p++; /* skip alpha */
+				} else if (alpha == 0) {
+					*q++ = 255;
+					*q++ = 255;
+					*q++ = 255;
+					p += 4;
+				} else {
+					*q++ = (*p++ * alpha) / 255 + one_minus_alpha;
+					*q++ = (*p++ * alpha) / 255 + one_minus_alpha;
+					*q++ = (*p++ * alpha) / 255 + one_minus_alpha;
+					p++; /* skip alpha */
+				}
+			}
+		}
+		     
+		hex_size = gnome_print_encode_hex (row, (guchar *) hex, w * out_ch);
 		problem |= fwrite (hex, sizeof (gchar), hex_size, ps2->buf);
 		problem |= gnome_print_ps2_puts (ps2, EOL);
 	}
 
 	g_free (hex);
 
+	if (out_ch != ch)
+		g_free (row);
+
 	problem |= gnome_print_ps2_puts (ps2, "Q" EOL);
 
 	return problem ? GNOME_PRINT_ERROR_UNKNOWN : GNOME_PRINT_OK;
