@@ -60,10 +60,7 @@
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-rbuf.h>
-#include <libgnomeprint/gnome-rfont.h>
-#include <libgnomeprint/gnome-font-private.h>
 #include <libgnomeprint/gnome-pgl.h>
-#include <libgnomeprint/gnome-pgl-private.h>
 #include <libgnomeprint/gp-gc-private.h>
 
 enum {
@@ -76,7 +73,9 @@ enum {
 	PROP_PAGE2BUF
 };
 
-struct _GnomePrintRBufPrivate {
+struct _GnomePrintRBuf {
+	GnomePrintContext parent;
+
 	guchar * pixels;
 	gint width;
 	gint height;
@@ -86,14 +85,13 @@ struct _GnomePrintRBufPrivate {
 	guint32 alpha : 1;
 };
 
+struct _GnomePrintRBufClass {
+	GnomePrintContextClass parent_class;
+};
+
 static void gpb_class_init (GnomePrintRBufClass *class);
 static void gpb_init (GnomePrintRBuf *rbuf);
-static void gpb_finalize (GObject *object);
 
-static gint gpb_beginpage (GnomePrintContext * pc, const guchar *name);
-static gint gpb_showpage (GnomePrintContext * pc);
-
-static gint gpb_clip (GnomePrintContext * pc, const ArtBpath *bpath, ArtWindRule rule);
 static gint gpb_fill (GnomePrintContext * pc, const ArtBpath *bpath, ArtWindRule rule);
 static gint gpb_stroke (GnomePrintContext * pc, const ArtBpath *bpath);
 
@@ -101,10 +99,8 @@ static gint gpb_image (GnomePrintContext * pc, const gdouble *affine, const guch
 
 static gint gpb_glyphlist (GnomePrintContext * pc, const gdouble *affine, GnomeGlyphList *gl);
 
-static gint gpb_close (GnomePrintContext * pc);
-
-static void gp_svp_uncross_to_render (GnomePrintContext * pc, const ArtSVP * svp, ArtWindRule rule);
-static void gp_vpath_to_render (GnomePrintContext * pc, const ArtBpath * bpath, ArtWindRule rule);
+static void gp_svp_uncross_to_render (GnomePrintRBuf *rbuf, const ArtSVP * svp, ArtWindRule rule);
+static void gp_vpath_to_render (GnomePrintRBuf *rbuf, const ArtBpath * bpath, ArtWindRule rule);
 
 static void gp_render_silly_rgba (GnomePrintContext * pc, const gdouble *affine, const guchar * px, gint w, gint h, gint rowstride);
 
@@ -145,22 +141,22 @@ gnome_print_rbuf_get_property (GObject *object, guint prop_id, GValue *value,
 
 	switch (prop_id) {
 	case PROP_WIDTH:
-		g_value_set_uint (value, r->priv->width);
+		g_value_set_uint (value, r->width);
 		break;
 	case PROP_HEIGHT:
-		g_value_set_uint (value, r->priv->height);
+		g_value_set_uint (value, r->height);
 		break;
 	case PROP_ROWSTRIDE:
-		g_value_set_uint (value, r->priv->rowstride);
+		g_value_set_uint (value, r->rowstride);
 		break;
 	case PROP_ALPHA:
-		g_value_set_boolean (value, r->priv->alpha);
+		g_value_set_boolean (value, r->alpha);
 		break;
 	case PROP_PIXELS:
-		g_value_set_pointer (value, r->priv->pixels);
+		g_value_set_pointer (value, r->pixels);
 		break;
 	case PROP_PAGE2BUF:
-		g_value_set_pointer (value, r->priv->page2buf);
+		g_value_set_pointer (value, r->page2buf);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -176,23 +172,23 @@ gnome_print_rbuf_set_property (GObject *object, guint prop_id,
 
 	switch (prop_id) {
 	case PROP_WIDTH:
-		r->priv->width = g_value_get_uint (value);
+		r->width = g_value_get_uint (value);
 		break;
 	case PROP_HEIGHT:
-		r->priv->height = g_value_get_uint (value);
+		r->height = g_value_get_uint (value);
 		break;
 	case PROP_ROWSTRIDE:
-		r->priv->rowstride = g_value_get_uint (value);
+		r->rowstride = g_value_get_uint (value);
 		break;
 	case PROP_ALPHA:
-		r->priv->alpha = g_value_get_boolean (value);
+		r->alpha = g_value_get_boolean (value);
 		break;
 	case PROP_PIXELS:
-		g_free (r->priv->pixels);
-		r->priv->pixels = g_value_get_pointer (value);
+		g_free (r->pixels);
+		r->pixels = g_value_get_pointer (value);
 		break;
 	case PROP_PAGE2BUF:
-		memcpy (r->priv->page2buf, g_value_get_pointer (value),
+		memcpy (r->page2buf, g_value_get_pointer (value),
 				sizeof (gdouble) * 6);
 		break;
 	default:
@@ -212,18 +208,13 @@ gpb_class_init (GnomePrintRBufClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = gpb_finalize;
 	object_class->get_property = gnome_print_rbuf_get_property;
 	object_class->set_property = gnome_print_rbuf_set_property;
 
-	pc_class->beginpage = gpb_beginpage;
-	pc_class->showpage = gpb_showpage;
-	pc_class->clip = gpb_clip;
 	pc_class->fill = gpb_fill;
 	pc_class->stroke = gpb_stroke;
 	pc_class->image = gpb_image;
 	pc_class->glyphlist = gpb_glyphlist;
-	pc_class->close = gpb_close;
 
 	g_object_class_install_property (object_class, PROP_WIDTH,
 			g_param_spec_uint ("width", _("Width"), _("Width"),
@@ -248,77 +239,17 @@ gpb_class_init (GnomePrintRBufClass *klass)
 static void
 gpb_init (GnomePrintRBuf * rbuf)
 {
-	rbuf->priv = g_new (GnomePrintRBufPrivate, 1);
-
-	rbuf->priv->pixels = NULL;
-	art_affine_identity (rbuf->priv->page2buf);
-}
-
-static void
-gpb_finalize (GObject *object)
-{
-	GnomePrintRBuf * rbuf;
-
-	rbuf = GNOME_PRINT_RBUF (object);
-
-	if (rbuf->priv) {
-		g_free (rbuf->priv);
-	}
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-/*
- * Beginpage
- *
- * fixme: Currently we simply clear rbuffer
- *
- * showpage, close - do nothing
- */
-
-static gint
-gpb_beginpage (GnomePrintContext *pc, const guchar *name)
-{
-	return GNOME_PRINT_OK;
-}
-
-static gint
-gpb_showpage (GnomePrintContext * pc)
-{
-	return GNOME_PRINT_OK;
-}
-
-static gint
-gpb_clip (GnomePrintContext * pc, const ArtBpath *bpath, ArtWindRule rule)
-{
-	/* FIXME: Why are we not clipping? (Chema) */
-#if 0
-	GnomePrintRBuf * rbuf;
-
-	rbuf = GNOME_PRINT_RBUF (pc);
-
-	if (rule == ART_WIND_RULE_NONZERO) {
-		gp_gc_clip (pc->gc);
-	} else {
-		gp_gc_eoclip (pc->gc);
-	}
-#endif
-
-	return GNOME_PRINT_OK;
+	art_affine_identity (rbuf->page2buf);
 }
 
 static gint
 gpb_fill (GnomePrintContext * pc, const ArtBpath *bpath, ArtWindRule rule)
 {
-	GnomePrintRBuf * rbuf;
-	GnomePrintRBufPrivate * rbp;
+	GnomePrintRBuf *rbuf = GNOME_PRINT_RBUF (pc);
 	ArtBpath * abp;
 
-	rbuf = GNOME_PRINT_RBUF (pc);
-	rbp = rbuf->priv;
-
-	abp = art_bpath_affine_transform (bpath, rbp->page2buf);
-	gp_vpath_to_render (pc, abp, rule);
+	abp = art_bpath_affine_transform (bpath, rbuf->page2buf);
+	gp_vpath_to_render (rbuf, abp, rule);
 	art_free (abp);
 
 	return 1;
@@ -327,18 +258,14 @@ gpb_fill (GnomePrintContext * pc, const ArtBpath *bpath, ArtWindRule rule)
 static gint
 gpb_stroke (GnomePrintContext * pc, const ArtBpath *bpath)
 {
-	GnomePrintRBuf * rbuf;
-	GnomePrintRBufPrivate * rbp;
+	GnomePrintRBuf *rbuf = GNOME_PRINT_RBUF (pc);
 	ArtBpath * abp;
 	ArtVpath * vpath, * pvpath;
 	ArtSVP * svp;
 	const ArtVpathDash * dash;
 	gdouble linewidth;
 
-	rbuf = GNOME_PRINT_RBUF (pc);
-	rbp = rbuf->priv;
-
-	abp = art_bpath_affine_transform (bpath, rbp->page2buf);
+	abp = art_bpath_affine_transform (bpath, rbuf->page2buf);
 	vpath = art_bez_path_to_vec (abp, 0.25);
 	art_free (abp);
 
@@ -368,7 +295,7 @@ gpb_stroke (GnomePrintContext * pc, const ArtBpath *bpath)
 	g_assert (svp != NULL);
 	art_free (pvpath);
 
-	gp_svp_uncross_to_render (pc, svp, ART_WIND_RULE_NONZERO);
+	gp_svp_uncross_to_render (rbuf, svp, ART_WIND_RULE_NONZERO);
 
 	art_svp_free (svp);
 
@@ -378,15 +305,10 @@ gpb_stroke (GnomePrintContext * pc, const ArtBpath *bpath)
 static gint
 gpb_image (GnomePrintContext * pc, const gdouble *affine, const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
-	GnomePrintRBuf * rbuf;
-	GnomePrintRBufPrivate * rbp;
 	art_u8 * ib, * ipd;
 	const art_u8 * ips;
 	gint x, y;
 
-	rbuf = GNOME_PRINT_RBUF (pc);
-	rbp = rbuf->priv;
-
 	if (ch == 1) {
 		ib = g_new (art_u8, w * h * 4);
 		for (y = 0; y < h; y++) {
@@ -422,53 +344,39 @@ gpb_image (GnomePrintContext * pc, const gdouble *affine, const guchar *px, gint
 	return 1;
 }
 
-/* fixme: */
-
 static gint
 gpb_glyphlist (GnomePrintContext * pc, const gdouble *affine, GnomeGlyphList *gl)
 {
-	GnomePrintRBuf *rbuf;
+	GnomePrintRBuf *rbuf = GNOME_PRINT_RBUF (pc);
 	GnomePosGlyphList *pgl;
 	gdouble t[6];
 
-	rbuf = (GnomePrintRBuf *) pc;
-
-	art_affine_multiply (t, affine, rbuf->priv->page2buf);
+	art_affine_multiply (t, affine, rbuf->page2buf);
 
 	pgl = gnome_pgl_from_gl (gl, t, 0);
 
-	if (rbuf->priv->alpha) {
-		gnome_rfont_render_pgl_rgba8 (pgl, 0, 0, rbuf->priv->pixels,
-					      rbuf->priv->width, rbuf->priv->height, rbuf->priv->rowstride, 0);
-	} else {
-		gnome_rfont_render_pgl_rgb8 (pgl, 0, 0, rbuf->priv->pixels,
-					     rbuf->priv->width, rbuf->priv->height, rbuf->priv->rowstride, 0);
-	}
+	if (rbuf->alpha)
+		gnome_pgl_render_rgba8 (pgl, 0, 0, rbuf->pixels,
+				rbuf->width, rbuf->height, rbuf->rowstride, 0);
+	else
+		gnome_pgl_render_rgb8 (pgl, 0, 0, rbuf->pixels,
+				rbuf->width, rbuf->height, rbuf->rowstride, 0);
 
 	gnome_pgl_destroy (pgl);
 
 	return 1;
 }
 
-static gint
-gpb_close (GnomePrintContext * pc)
-{
-	return 1;
-}
-
 /* Private helpers */
 
 static void
-gp_svp_uncross_to_render (GnomePrintContext * pc, const ArtSVP * svp, ArtWindRule rule)
+gp_svp_uncross_to_render (GnomePrintRBuf *rbuf, const ArtSVP * svp, ArtWindRule rule)
 {
-	GnomePrintRBufPrivate * rbp;
 	ArtSVP * svp1, * svp2;
 
-	g_assert (pc != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_RBUF (rbuf));
 	g_assert (svp != NULL);
 
-	rbp = GNOME_PRINT_RBUF (pc)->priv;
-
 	svp2 = art_svp_uncross ((ArtSVP *) svp);
 	g_assert (svp2 != NULL);
 
@@ -476,24 +384,24 @@ gp_svp_uncross_to_render (GnomePrintContext * pc, const ArtSVP * svp, ArtWindRul
 	g_assert (svp1 != NULL);
 	art_svp_free (svp2);
 
-	if (gp_gc_has_clipsvp (pc->gc)) {
-		svp2 = art_svp_intersect (svp1, gp_gc_get_clipsvp (pc->gc));
+	if (gp_gc_has_clipsvp (GNOME_PRINT_CONTEXT (rbuf)->gc)) {
+		svp2 = art_svp_intersect (svp1, gp_gc_get_clipsvp (GNOME_PRINT_CONTEXT (rbuf)->gc));
 		g_assert (svp2 != NULL);
 		art_svp_free (svp1);
 		svp1 = svp2;
 	}
 
-	if (rbp->alpha) {
+	if (rbuf->alpha) {
 		gnome_print_art_rgba_svp_alpha (svp1,
-			0, 0, rbp->width, rbp->height,
-			gp_gc_get_rgba (pc->gc),
-			rbp->pixels, rbp->rowstride,
+			0, 0, rbuf->width, rbuf->height,
+			gp_gc_get_rgba (GNOME_PRINT_CONTEXT (rbuf)->gc),
+			rbuf->pixels, rbuf->rowstride,
 			NULL);
 	} else {
 		art_rgb_svp_alpha (svp1,
-			0, 0, rbp->width, rbp->height,
-			gp_gc_get_rgba (pc->gc),
-			rbp->pixels, rbp->rowstride,
+			0, 0, rbuf->width, rbuf->height,
+			gp_gc_get_rgba (GNOME_PRINT_CONTEXT (rbuf)->gc),
+			rbuf->pixels, rbuf->rowstride,
 			NULL);
 	}
 
@@ -501,17 +409,14 @@ gp_svp_uncross_to_render (GnomePrintContext * pc, const ArtSVP * svp, ArtWindRul
 }
 
 static void
-gp_vpath_to_render (GnomePrintContext * pc, const ArtBpath * bpath, ArtWindRule rule)
+gp_vpath_to_render (GnomePrintRBuf *rbuf, const ArtBpath * bpath, ArtWindRule rule)
 {
-	GnomePrintRBufPrivate * rbp;
 	ArtVpath * vpath1, * vpath2;
 	ArtSVP * svp;
 
-	g_assert (pc != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_RBUF (rbuf));
 	g_assert (bpath != NULL);
 
-	rbp = GNOME_PRINT_RBUF (pc)->priv;
-
 	vpath1 = art_bez_path_to_vec (bpath, 0.25);
 	g_assert (vpath1 != NULL);
 
@@ -523,7 +428,7 @@ gp_vpath_to_render (GnomePrintContext * pc, const ArtBpath * bpath, ArtWindRule
 	g_assert (svp != NULL);
 	art_free (vpath2);
 
-	gp_svp_uncross_to_render (pc, svp, rule);
+	gp_svp_uncross_to_render (rbuf, svp, rule);
 
 	art_svp_free (svp);
 }
@@ -551,13 +456,13 @@ gp_render_silly_rgba (GnomePrintContext * pc, const gdouble *affine, const gucha
 
 	rbuf = GNOME_PRINT_RBUF (pc);
 
-	art_affine_multiply (a, affine, rbuf->priv->page2buf);
+	art_affine_multiply (a, affine, rbuf->page2buf);
 	vpath = art_vpath_affine_transform (vp, a);
 
 	art_affine_scale (imga, 1.0 / w, -1.0 / h);
 	imga[5] = 1.0;
 	art_affine_multiply (a, imga, affine);
-	art_affine_multiply (a, a, rbuf->priv->page2buf);
+	art_affine_multiply (a, a, rbuf->page2buf);
 
 	pvpath = art_vpath_perturb (vpath);
 	art_free (vpath);
@@ -578,8 +483,8 @@ gp_render_silly_rgba (GnomePrintContext * pc, const gdouble *affine, const gucha
 	art_drect_svp (&bbox, svp1);
 
 	pbox.x0 = pbox.y0 = 0.0;
-	pbox.x1 = rbuf->priv->width;
-	pbox.y1 = rbuf->priv->height;
+	pbox.x1 = rbuf->width;
+	pbox.y1 = rbuf->height;
 
 	art_drect_intersect (&bbox, &bbox, &pbox);
 
@@ -655,15 +560,15 @@ gp_render_silly_rgba (GnomePrintContext * pc, const gdouble *affine, const gucha
 
 	/* Render */
 
-	if (rbuf->priv->alpha) {
-		gnome_print_art_rgba_rgba_affine (rbuf->priv->pixels,
-			0, 0, rbuf->priv->width, rbuf->priv->height, rbuf->priv->rowstride,
+	if (rbuf->alpha) {
+		gnome_print_art_rgba_rgba_affine (rbuf->pixels,
+			0, 0, rbuf->width, rbuf->height, rbuf->rowstride,
 			ibuf, bw, bh, bw * 4,
 			ba,
 			ART_FILTER_NEAREST, NULL);
 	} else {
-		art_rgb_rgba_affine (rbuf->priv->pixels,
-			0, 0, rbuf->priv->width, rbuf->priv->height, rbuf->priv->rowstride,
+		art_rgb_rgba_affine (rbuf->pixels,
+			0, 0, rbuf->width, rbuf->height, rbuf->rowstride,
 			ibuf, bw, bh, bw * 4,
 			ba,
 			ART_FILTER_NEAREST, NULL);
