@@ -254,26 +254,26 @@ get_paper_text (ppd_file_t const *ppd, ppd_size_t const *size)
 				len = strlen (option->choices[k].text);
 				if (ppd->lang_encoding) {
 					result = g_convert (option->choices[k].text, len,
-							    "UTF8", ppd->lang_encoding,
+							    "UTF-8", ppd->lang_encoding,
 							    NULL, NULL, NULL);
 					if (result != NULL)
 						return result;
 					warn_of_unknown_encoding (ppd->lang_encoding);
 				}
 				return g_convert (option->choices[k].text,  len,
-						  "UTF8",	"CSISOLatin1",
+						  "UTF-8",	"CSISOLatin1",
 						  NULL, NULL, NULL);
 			}
 		}
 	}
 
 	len = strlen (size->name);
-	result = g_convert (size->name,  len, "UTF8", ppd->lang_encoding,
+	result = g_convert (size->name,  len, "UTF-8", ppd->lang_encoding,
 			    NULL, NULL, NULL);
 	if (result != NULL)
 		return result;
 	warn_of_unknown_encoding (ppd->lang_encoding);
-	return g_convert (size->name,  len, "UTF8", "CSISOLatin1",
+	return g_convert (size->name,  len, "UTF-8", "CSISOLatin1",
 			  NULL, NULL, NULL);
 }
 
